searchData={"items":[{"type":"module","title":"Styler","doc":"Styler is a formatter plugin with stronger opinions on code organization, multi-line defs and other code-style matters.","ref":"Styler.html"},{"type":"module","title":"Styler.AliasEnv","doc":"A datastructure for maintaining something like compiler alias state when traversing AST.\n\nNot anywhere as correct as what the compiler gives us, but close enough for open source work.\n\nA alias env is a map from an alias's `as` to its resolution in a context.\n\nGiven the ast for\n\n    alias Foo.Bar\n\nwe'd create the env:\n\n    %{:Bar => [:Foo, :Bar]}","ref":"Styler.AliasEnv.html"},{"type":"function","title":"Styler.AliasEnv.define/2","doc":"","ref":"Styler.AliasEnv.html#define/2"},{"type":"function","title":"Styler.AliasEnv.expand/2","doc":"","ref":"Styler.AliasEnv.html#expand/2"},{"type":"behaviour","title":"Styler.Style","doc":"A Style takes AST and returns a transformed version of that AST.\n\nBecause these transformations involve traversing trees (the \"T\" in \"AST\"), we wrap the AST in a structure\ncalled a Zipper to facilitate walking the trees.","ref":"Styler.Style.html"},{"type":"function","title":"Styler.Style.displace_comments/2","doc":"Set the line of all comments with `line` in `range_start..range_end` to instead have line `range_start`","ref":"Styler.Style.html#displace_comments/2"},{"type":"function","title":"Styler.Style.ensure_block_parent/1","doc":"Returns the current node (wrapped in a `__block__` if necessary) if it's a valid place to insert additional nodes","ref":"Styler.Style.html#ensure_block_parent/1"},{"type":"function","title":"Styler.Style.find_nearest_block/1","doc":"Returns a zipper focused on the nearest node where additional nodes can be inserted (a \"block\").\n\nThe nearest node is either the current node, an ancestor, or one of those two but wrapped in a new `:__block__` node.","ref":"Styler.Style.html#find_nearest_block/1"},{"type":"function","title":"Styler.Style.fix_line_numbers/2","doc":"\"Fixes\" the line numbers of nodes who have had their orders changed via sorting or other methods.\nThis \"fix\" simply ensures that comments don't get wrecked as part of us moving AST nodes willy-nilly.\n\nThe fix is rather naive, and simply enforces the following property on the code:\nA given node must have a line number less than the following node.\nEt voila! Comments behave much better.","ref":"Styler.Style.html#fix_line_numbers/2"},{"type":"function","title":"In Detail - Styler.Style.fix_line_numbers/2","doc":"For example, given document\n\n  1: defmodule ...\n  2: alias B\n  3: # this is foo\n  4: def foo ...\n  5: alias A\n\nSorting aliases the ast node for  would put `alias A` (line 5) before `alias B` (line 2).\n\n  1: defmodule ...\n  5: alias A\n  2: alias B\n  3: # this is foo\n  4: def foo ...\n\nElixir's document algebra would then encounter `line: 5` and immediately dump all comments with `line <= 5`,\nmeaning after running through the formatter we'd end up with\n\n  1: defmodule\n  2: # hi\n  3: # this is foo\n  4: alias A\n  5: alias B\n  6:\n  7: def foo ...\n\nThis function fixes that by seeing that `alias A` has a higher line number than its following sibling `alias B` and so\nupdates `alias A`'s line to be preceding `alias B`'s line.\n\nRunning the results of this function through the formatter now no longer dumps the comments prematurely\n\n  1: defmodule ...\n  2: alias A\n  3: alias B\n  4: # this is foo\n  5: def foo ...","ref":"Styler.Style.html#fix_line_numbers/2-in-detail"},{"type":"function","title":"Styler.Style.max_line/1","doc":"","ref":"Styler.Style.html#max_line/1"},{"type":"function","title":"Styler.Style.reset_newlines/1","doc":"Takes a list of nodes and clumps them up, setting `end_of_expression: [newlines: x]` to 1 for all but the final node,\nwhich gets 2 instead, (hopefully!) creating an empty line before whatever follows.","ref":"Styler.Style.html#reset_newlines/1"},{"type":"function","title":"Styler.Style.reset_newlines/2","doc":"","ref":"Styler.Style.html#reset_newlines/2"},{"type":"callback","title":"Styler.Style.run/2","doc":"`run` will be used with `Zipper.traverse_while/3`, meaning it will be executed on every node of the AST.\n\nYou can skip traversing parts of the tree by returning a Zipper that's further along in the traversal, for example\nby calling `Zipper.skip(zipper)` to skip an entire subtree you know is of no interest to your Style.","ref":"Styler.Style.html#c:run/2"},{"type":"function","title":"Styler.Style.set_line/3","doc":"Recursively sets `:line` meta to `line`. Deletes `:newlines` unless `delete_lines: false` is passed","ref":"Styler.Style.html#set_line/3"},{"type":"function","title":"Styler.Style.shift_comments/2","doc":"Perform a series of shifts in a single pass.\n\nWhen shifting comments from block A to block B, naively using two passes of `shift_comments/3` would result\nin all comments ending up in either region A or region B (because A would move to B, then all B back to A)\nThis function exists to make sure that a comment is only moved once during the swap.","ref":"Styler.Style.html#shift_comments/2"},{"type":"function","title":"Styler.Style.shift_comments/3","doc":"Change the `line` of all comments with `line` in `range` by adding `delta` to it.\nA positive delta will move the lines further down a file, while a negative delta will move them up.","ref":"Styler.Style.html#shift_comments/3"},{"type":"function","title":"Styler.Style.shift_line/2","doc":"Recursively updates `:line` meta by adding `delta`","ref":"Styler.Style.html#shift_line/2"},{"type":"function","title":"Styler.Style.update_all_meta/2","doc":"Traverses an ast node, updating all nodes' meta with `meta_fun`","ref":"Styler.Style.html#update_all_meta/2"},{"type":"function","title":"Styler.Style.without_meta/1","doc":"","ref":"Styler.Style.html#without_meta/1"},{"type":"type","title":"Styler.Style.context/0","doc":"","ref":"Styler.Style.html#t:context/0"},{"type":"module","title":"Styler.Style.Blocks","doc":"Simple 1-1 rewrites all crammed into one module to make for more efficient traversals\n\nCredo Rules addressed:\n\n* Credo.Check.Consistency.ParameterPatternMatching\n* Credo.Check.Readability.LargeNumbers\n* Credo.Check.Readability.ParenthesesOnZeroArityDefs\n* Credo.Check.Readability.PreferImplicitTry\n* Credo.Check.Readability.WithSingleClause\n* Credo.Check.Refactor.CaseTrivialMatches\n* Credo.Check.Refactor.CondStatements\n* Credo.Check.Refactor.RedundantWithClauseResult\n* Credo.Check.Refactor.WithClauses","ref":"Styler.Style.Blocks.html"},{"type":"function","title":"Styler.Style.Blocks.run/2","doc":"","ref":"Styler.Style.Blocks.html#run/2"},{"type":"module","title":"Styler.Style.Configs","doc":"Orders `Config.config/2,3` stanzas in configuration files.\n\n- ordering is done only within immediate-sibling config statements\n- assignments are moved above the configuration blocks\n- any non `config/2,3` or assignment (`=/2`) calls mark the end of a sorting block.\n  this is support having conditional blocks (`if/case/cond`) and `import_config` stanzas between blocks\n\n#","ref":"Styler.Style.Configs.html"},{"type":"module","title":"Breakages - Styler.Style.Configs","doc":"If you configure the same values multiple times, Styler may swap their orders\n\n**Before**\n\n  line 04: config :foo, bar: :zab\n  line 40: config :foo, bar: :baz\n\n  # Application.fetch_env!(:foo)[:bar] => :baz\n\n**After**\n\n  line 04: config :foo, bar: :baz\n  line 05: config :foo, bar: :zab\n\n  # Application.fetch_env!(:foo)[:bar] => :zab\n\n**Fix**\n\nThe reason Styler sorts configuration is to help you noticed these duplicated configuration stanzas.\nDelete the duplicative/erroneous stanza and life will be good.","ref":"Styler.Style.Configs.html#module-breakages"},{"type":"function","title":"Styler.Style.Configs.run/2","doc":"","ref":"Styler.Style.Configs.html#run/2"},{"type":"function","title":"Styler.Style.Configs.set_lines/5","doc":"","ref":"Styler.Style.Configs.html#set_lines/5"},{"type":"module","title":"Styler.Style.Defs","doc":"Styles function heads so that they're as small as possible.\n\nThe goal is that a function head fits on a single line.\n\nThis isn't a Credo issue, and the formatter is fine with either approach. But Styler has opinions!\n\nEx:\n\nThis long declaration\n\n    def foo(%{\n      bar: baz\n    }) do\n      ...\n    end\n\nBecomes\n\n    def foo(%{bar: baz}) do\n      ...\n    end","ref":"Styler.Style.Defs.html"},{"type":"function","title":"Styler.Style.Defs.run/2","doc":"","ref":"Styler.Style.Defs.html#run/2"},{"type":"module","title":"Styler.Style.Deprecations","doc":"Transformations to soft or hard deprecations introduced on newer Elixir releases","ref":"Styler.Style.Deprecations.html"},{"type":"function","title":"Styler.Style.Deprecations.run/2","doc":"","ref":"Styler.Style.Deprecations.html#run/2"},{"type":"module","title":"Styler.Style.ModuleDirectives","doc":"Styles up module directives!\n\nThis Style will expand multi-aliases/requires/imports/use and sort the directive within its groups (except `use`s, which cannot be sorted)\nIt also adds a blank line after each directive group.","ref":"Styler.Style.ModuleDirectives.html"},{"type":"module","title":"Credo rules - Styler.Style.ModuleDirectives","doc":"Rewrites for the following Credo rules:\n\n  * `Credo.Check.Consistency.MultiAliasImportRequireUse` (force expansion)\n  * `Credo.Check.Readability.AliasOrder` (we sort `__MODULE__`, which credo doesn't)\n  * `Credo.Check.Readability.ModuleDoc` (adds `@moduledoc false` if missing. includes `*.exs` files)\n  * `Credo.Check.Readability.MultiAlias`\n  * `Credo.Check.Readability.StrictModuleLayout` (see section below for details)\n  * `Credo.Check.Readability.UnnecessaryAliasExpansion`\n  * `Credo.Check.Design.AliasUsage`\n\n#","ref":"Styler.Style.ModuleDirectives.html#module-credo-rules"},{"type":"module","title":"Strict Layout - Styler.Style.ModuleDirectives","doc":"Modules directives are sorted into the following order:\n\n  * `@shortdoc`\n  * `@moduledoc`\n  * `@behaviour`\n  * `use`\n  * `import`\n  * `alias`\n  * `require`\n  * everything else (unchanged)","ref":"Styler.Style.ModuleDirectives.html#module-strict-layout"},{"type":"function","title":"Styler.Style.ModuleDirectives.run/2","doc":"","ref":"Styler.Style.ModuleDirectives.html#run/2"},{"type":"module","title":"Styler.Style.Pipes","doc":"Styles pipes! In particular, don't make pipe chains of only one pipe, and some persnickety pipe chain start stuff.\n\nRewrites for the following Credo rules:\n\n  * Credo.Check.Readability.BlockPipe\n  * Credo.Check.Readability.OneArityFunctionInPipe\n  * Credo.Check.Readability.PipeIntoAnonymousFunctions\n  * Credo.Check.Readability.SinglePipe\n  * Credo.Check.Refactor.FilterCount\n  * Credo.Check.Refactor.MapInto\n  * Credo.Check.Refactor.MapJoin\n  * Credo.Check.Refactor.PipeChainStart, excluded_functions: [\"from\"]","ref":"Styler.Style.Pipes.html"},{"type":"function","title":"Styler.Style.Pipes.run/2","doc":"","ref":"Styler.Style.Pipes.html#run/2"},{"type":"module","title":"Styler.Style.SingleNode","doc":"Simple 1-1 rewrites all crammed into one module to make for more efficient traversals\n\nCredo Rules addressed:\n\n* Credo.Check.Consistency.ParameterPatternMatching\n* Credo.Check.Readability.LargeNumbers\n* Credo.Check.Readability.ParenthesesOnZeroArityDefs\n* Credo.Check.Readability.PreferImplicitTry\n* Credo.Check.Readability.StringSigils\n* Credo.Check.Readability.WithSingleClause\n* Credo.Check.Refactor.CaseTrivialMatches\n* Credo.Check.Refactor.CondStatements\n* Credo.Check.Refactor.RedundantWithClauseResult\n* Credo.Check.Refactor.WithClauses","ref":"Styler.Style.SingleNode.html"},{"type":"function","title":"Styler.Style.SingleNode.run/2","doc":"","ref":"Styler.Style.SingleNode.html#run/2"},{"type":"module","title":"Styler.Zipper","doc":"Implements a Zipper for the Elixir AST based on Gérard Huet [Functional pearl: the\nzipper](https://www.st.cs.uni-saarland.de/edu/seminare/2005/advanced-fp/docs/huet-zipper.pdf) paper and\nClojure's `clojure.zip` API.\n\nA zipper is a data structure that represents a location in a tree from the\nperspective of the current node, also called *focus*. It is represented by a\n2-tuple where the first element is the focus and the second element is the\nmetadata/context. The metadata is `nil` when the focus is the topmost node","ref":"Styler.Zipper.html"},{"type":"function","title":"Styler.Zipper.any?/2","doc":"Traverses `zipper`, returning true when `fun.(Zipper.node(zipper))` is truthy, or false otherwise","ref":"Styler.Zipper.html#any?/2"},{"type":"function","title":"Styler.Zipper.append_child/2","doc":"Inserts the item as the rightmost child of the node at this zipper,\nwithout moving.","ref":"Styler.Zipper.html#append_child/2"},{"type":"function","title":"Styler.Zipper.children/1","doc":"Returns a list of children of the node.","ref":"Styler.Zipper.html#children/1"},{"type":"function","title":"Styler.Zipper.down/1","doc":"Returns the zipper of the leftmost child of the node at this zipper, or\nnil if no there's no children.","ref":"Styler.Zipper.html#down/1"},{"type":"function","title":"Styler.Zipper.find/3","doc":"Returns a zipper to the node that satisfies the predicate function, or `nil`\nif none is found.\n\nThe optional second parameters specifies the `direction`, defaults to\n`:next`.","ref":"Styler.Zipper.html#find/3"},{"type":"function","title":"Styler.Zipper.insert_child/2","doc":"Inserts the item as the leftmost child of the node at this zipper,\nwithout moving.","ref":"Styler.Zipper.html#insert_child/2"},{"type":"function","title":"Styler.Zipper.insert_left/2","doc":"Inserts the item as the left sibling of the node at this zipper, without\nmoving. Raises an `ArgumentError` when attempting to insert a sibling at the\ntop level.","ref":"Styler.Zipper.html#insert_left/2"},{"type":"function","title":"Styler.Zipper.insert_right/2","doc":"Inserts the item as the right sibling of the node at this zipper, without\nmoving. Raises an `ArgumentError` when attempting to insert a sibling at the\ntop level.","ref":"Styler.Zipper.html#insert_right/2"},{"type":"function","title":"Styler.Zipper.insert_siblings/2","doc":"Inserts many siblings to the right.\n\nEquivalent to\n\n    Enum.reduce(siblings, zipper, &Zipper.insert_right(&2, &1))","ref":"Styler.Zipper.html#insert_siblings/2"},{"type":"function","title":"Styler.Zipper.left/1","doc":"Returns the zipper of the left sibling of the node at this zipper, or nil.","ref":"Styler.Zipper.html#left/1"},{"type":"function","title":"Styler.Zipper.leftmost/1","doc":"Returns the leftmost sibling of the node at this zipper, or itself.","ref":"Styler.Zipper.html#leftmost/1"},{"type":"function","title":"Styler.Zipper.next/1","doc":"Returns the following zipper in depth-first pre-order.","ref":"Styler.Zipper.html#next/1"},{"type":"function","title":"Styler.Zipper.node/1","doc":"Returns the node at the zipper.","ref":"Styler.Zipper.html#node/1"},{"type":"function","title":"Styler.Zipper.prepend_siblings/2","doc":"Inserts many siblings to the left.\n\nEquivalent to\n\n    Enum.reduce(siblings, zipper, &Zipper.insert_left(&2, &1))","ref":"Styler.Zipper.html#prepend_siblings/2"},{"type":"function","title":"Styler.Zipper.prev/1","doc":"Returns the previous zipper in depth-first pre-order.\nReturns nil if the tree is already at the top.","ref":"Styler.Zipper.html#prev/1"},{"type":"function","title":"Styler.Zipper.remove/1","doc":"Removes the node at the zipper, returning the zipper that would have preceded\nit in a depth-first walk.","ref":"Styler.Zipper.html#remove/1"},{"type":"function","title":"Styler.Zipper.replace/2","doc":"Replaces the current node in the zipper with a new node.","ref":"Styler.Zipper.html#replace/2"},{"type":"function","title":"Styler.Zipper.replace_children/2","doc":"Returns the zipper with the current children of the node replaced with `children`","ref":"Styler.Zipper.html#replace_children/2"},{"type":"function","title":"Styler.Zipper.right/1","doc":"Returns the zipper of the right sibling of the node at this zipper, or nil.","ref":"Styler.Zipper.html#right/1"},{"type":"function","title":"Styler.Zipper.rightmost/1","doc":"Returns the rightmost sibling of the node at this zipper, or itself.","ref":"Styler.Zipper.html#rightmost/1"},{"type":"function","title":"Styler.Zipper.root/1","doc":"Walks the zipper all the way up and returns the root node.","ref":"Styler.Zipper.html#root/1"},{"type":"function","title":"Styler.Zipper.skip/2","doc":"Returns the zipper of the right sibling of the node at this zipper, or the\nnext zipper when no right sibling is available.\n\nThis allows to skip subtrees while traversing the siblings of a node.\n\nThe optional second parameters specifies the `direction`, defaults to\n`:next`.\n\nIf no right/left sibling is available, this function returns the same value as\n`next/1`/`prev/1`.\n\nThe function `skip/1` behaves like the `:skip` in `traverse_while/2` and\n`traverse_while/3`.","ref":"Styler.Zipper.html#skip/2"},{"type":"function","title":"Styler.Zipper.top/1","doc":"Walks the zipper all the way up and returns the top zipper.","ref":"Styler.Zipper.html#top/1"},{"type":"function","title":"Styler.Zipper.traverse/2","doc":"Traverses the tree in depth-first pre-order calling the given function for\neach node.\n\nIf the zipper is not at the top, just the subtree will be traversed.\n\nThe function must return a zipper.","ref":"Styler.Zipper.html#traverse/2"},{"type":"function","title":"Styler.Zipper.traverse/3","doc":"Traverses the tree in depth-first pre-order calling the given function for\neach node with an accumulator.\n\nIf the zipper is not at the top, just the subtree will be traversed.","ref":"Styler.Zipper.html#traverse/3"},{"type":"function","title":"Styler.Zipper.traverse_while/2","doc":"Traverses the tree in depth-first pre-order calling the given function for\neach node.\n\nThe traversing will continue if the function returns `{:cont, zipper}`,\nskipped for `{:skip, zipper}` and halted for `{:halt, zipper}`\n\nIf the zipper is not at the top, just the subtree will be traversed.\n\nThe function must return a zipper.","ref":"Styler.Zipper.html#traverse_while/2"},{"type":"function","title":"Styler.Zipper.traverse_while/3","doc":"Traverses the tree in depth-first pre-order calling the given function for\neach node with an accumulator.\n\nThe traversing will continue if the function returns `{:cont, zipper, acc}`,\nskipped for `{:skip, zipper, acc}` and halted for `{:halt, zipper, acc}`\n\nIf the zipper is not at the top, just the subtree will be traversed.","ref":"Styler.Zipper.html#traverse_while/3"},{"type":"function","title":"Styler.Zipper.up/1","doc":"Returns the zipper of the parent of the node at this zipper, or nil if at the\ntop.","ref":"Styler.Zipper.html#up/1"},{"type":"function","title":"Styler.Zipper.update/2","doc":"Replaces the current node in the zipper with the result of applying `fun` to\nthe node.","ref":"Styler.Zipper.html#update/2"},{"type":"function","title":"Styler.Zipper.zip/1","doc":"Creates a zipper from a tree node.","ref":"Styler.Zipper.html#zip/1"},{"type":"type","title":"Styler.Zipper.command/0","doc":"","ref":"Styler.Zipper.html#t:command/0"},{"type":"opaque","title":"Styler.Zipper.path/0","doc":"","ref":"Styler.Zipper.html#t:path/0"},{"type":"type","title":"Styler.Zipper.t/0","doc":"","ref":"Styler.Zipper.html#t:t/0"},{"type":"type","title":"Styler.Zipper.tree/0","doc":"","ref":"Styler.Zipper.html#t:tree/0"},{"type":"type","title":"Styler.Zipper.zipper/0","doc":"","ref":"Styler.Zipper.html#t:zipper/0"},{"type":"exception","title":"Styler.StyleError","doc":"Wraps errors raised by Styles during tree traversal.","ref":"Styler.StyleError.html"},{"type":"function","title":"Styler.StyleError.message/1","doc":"","ref":"Styler.StyleError.html#message/1"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\n**Note** Styler's only public API is its usage as a formatter plugin. While you're welcome to play with its internals,\nthey can and will change without that change being reflected in Styler's semantic version.","ref":"changelog.html"},{"type":"extras","title":"main - Changelog","doc":"","ref":"changelog.html#main"},{"type":"extras","title":"1.0.0-rc.1 - Changelog","doc":"#","ref":"changelog.html#1-0-0-rc-1"},{"type":"extras","title":"Improvements - Changelog","doc":"* Lots of documentation added. Nearly done and ready for 1.0.0.\n* `Enum.into(x, [])` => `Enum.to_list(x)`\n* `Enum.into(x, [], mapper)` => `Enum.map(x, mapper)`\n* `a |> Enum.map(m) |> Enum.join()` to `map_join(a, m)`. we already did this for `join/2`, but missed the case for `join/1`","ref":"changelog.html#improvements"},{"type":"extras","title":"1.0.0-rc.0 - Changelog","doc":"At this point, 1.0.0 feels feature complete. Two things remains for a full release:\n\n1. feedback!\n2. documentation overhaul! [monitor progress here](https://github.com/adobe/elixir-styler/pull/166)\n\n#","ref":"changelog.html#1-0-0-rc-0"},{"type":"extras","title":"Improvements - Changelog","doc":"Styler's two biggest outstanding bugs have been fixed, both related to compilation breaking during module directive organization. One was references to aliases being moved above where the aliases were declared, and the other was similarly module directives being moved after their uses in module directives.\n\nIn both cases, Styler is now smart enough to auto-apply the fixes we recommended in the old Readme.\n\nOther than that, a slew of powerful new features have been added, the neatest one (in the author's opinion anyways) being Alias Lifting.\n\n##","ref":"changelog.html#improvements"},{"type":"extras","title":"Alias Lifting - Changelog","doc":"Along the lines of `Credo.Check.Design.AliasUsage`, Styler now \"lifts\" deeply nested aliases (depth >= 3, ala `A.B.C....`) that are used more than once.\n\nPut plainly, this code:\n\n```elixir\ndefmodule A do\n  def lift_me() do\n    A.B.C.foo()\n    A.B.C.baz()\n  end\nend\n```\n\nwill become\n\n```elixir\ndefmodule A do\n  @moduledoc false\n  alias A.B.C\n\n  def lift_me do\n    C.foo()\n    C.baz()\n  end\nend\n```\n\nTo exclude modules ending in `.Foo` from being lifted, add `styler: [alias_lifting_exclude: [Foo]]` to your `.formatter.exs`\n\n##","ref":"changelog.html#alias-lifting"},{"type":"extras","title":"Module Attribute Lifting - Changelog","doc":"A long outstanding breakage of a first pass with Styler was breaking directives that relied on module attributes which Styler moved _after_ their uses. Styler now detects these potential breakages and automatically applies our suggested fix, which is creating a variable before the module. This usually happened when folks were using a library that autogenerated their moduledocs for them.\n\nIn code, this module:\n\n```elixir\ndefmodule MyGreatLibrary do\n  @library_options [...]\n  @moduledoc make_pretty_docs(@library_options)\n  use OptionsMagic, my_opts: @library_options\n\n  ...\nend\n```\n\nWill now be styled like so:\n\n```elixir\nlibrary_options = [...]\n\ndefmodule MyGreatLibrary do\n  @moduledoc make_pretty_docs(library_options)\n  use OptionsMagic, my_opts: unquote(library_options)\n\n  @library_options library_options\n\n  ...\nend\n```\n\n##","ref":"changelog.html#module-attribute-lifting"},{"type":"extras","title":"Mix Config File Organization - Changelog","doc":"Styler now organizes `Mix.Config.config/2,3` stanzas according to erlang term sorting. This helps manage large configuration files, removing the \"where should I put this\" burden from developers AND helping find duplicated configuration stanzas.\n\nSee the moduledoc for `Styler.Style.Configs` for more.\n\n##","ref":"changelog.html#mix-config-file-organization"},{"type":"extras","title":"Everything Else - Changelog","doc":"* `if`/`unless`: invert if and unless with `!=` or `!==`, like we do for `!` and `not` #132\n* `@derive`: move `@derive` before `defstruct|schema|embedded_schema` declarations (fixes compiler warning!) #134\n* strings: rewrite double-quoted strings to use `~s` when there's 4+ escaped double-quotes\n  (`\"\\\"\\\"\\\"\\\"\"` -> `~s(\"\"\"\")`) (`Credo.Check.Readability.StringSigils`) #146\n* `Map.drop(foo, [single_key])` => `Map.delete(foo, single_key)` #161 (also in pipes)\n* `Keyword.drop(foo, [single_key])` => `Keyword.delete(foo, single_key)` #161 (also in pipes)\n* `lhs |> Enum.reverse() |> Kernel.++(enum)` => `lhs |> Enum.reverse(enum)`\n\n#","ref":"changelog.html#everything-else"},{"type":"extras","title":"Fixes - Changelog","doc":"* `alias`: expands aliases when moving an alias after another directive that relied on it (#137)\n* module directives: various fixes for unreported obscure crashes\n* pipes: fix a comment-shifting scenario when unpiping\n* `Timex.now/1` will no longer be rewritten to `DateTime.now!/1` due to Timex accepting a wider domain of \"timezones\" than the stdlib (#145, h/t @ivymarkwell)\n* `with`: skip nodes which (unexpectedly) do not contain a `do` body (#158, h/t @DavidB59)\n* `then(&fun/1)`: fix false positives on arithmetic `&1 + x / 1` (#164, h/t @aenglisc)\n\n#","ref":"changelog.html#fixes"},{"type":"extras","title":"Breaking Changes - Changelog","doc":"* drop support for elixir `1.14`\n* ModuleDirectives: group callback attributes (`before_compile after_compile after_verify`) with nondirectives (previously, were grouped with `use`, their relative order maintained). to keep the desired behaviour, you can make new `use` macros that wrap these callbacks. Apologies if this makes using Styler untenable for your codebase, but it's probably not a good tool for macro-heavy libraries.\n* sorting configs for the first time can change your configuration; see `Styler.Style.Configs` moduledoc for more","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"v0.11.9 - Changelog","doc":"#","ref":"changelog.html#v0-11-9"},{"type":"extras","title":"Improvements - Changelog","doc":"* pipes: check for `Stream.foo` equivalents to `Enum.foo` in a few more cases\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* pipes: `|> then(&(&1 op y))` rewrites with `|> Kernel.op(y)` as long as the operator is defined in `Kernel`; skips the rewrite otherwise (h/t @kerryb for the report & @saveman71 for the fix)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.11.8 - Changelog","doc":"Two releases in one day!? @koudelka made too good a point about `Map.new` not being special...\n\n#","ref":"changelog.html#v0-11-8"},{"type":"extras","title":"Improvements - Changelog","doc":"* pipes: treat `MapSet.new` and `Keyword.new` the same way we do `Map.new` (h/t @koudelka)\n* pipes: treat `Stream.map` the same as `Enum.map` when piped `|> Enum.into`","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.11.7 - Changelog","doc":"#","ref":"changelog.html#v0-11-7"},{"type":"extras","title":"Improvements - Changelog","doc":"* deprecations: `~R` -> `~r`, `Date.range/2` -> `Date.range/3` with decreasing dates (h/t @milmazz)\n* if: rewrite `if not x, do: y` => `unless x, do: y`\n* pipes: `|> Enum.map(foo) |> Map.new()` => `|> Map.new(foo)`\n* pipes: remove unnecessary `then/2` on named function captures: `|> then(&foo/1)` => `|> foo()`, `|> then(&foo(&1, ...))` => `|> foo(...)` (thanks to @tfiedlerdejanze for the idea + impl!)","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.11.6 - Changelog","doc":"#","ref":"changelog.html#v0-11-6"},{"type":"extras","title":"Fixes - Changelog","doc":"* directives: maintain order of module compilation callbacks (`@before_compile` etc) relative to `use` statements (Closes #120, h/t @frankdugan3)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.11.5 - Changelog","doc":"#","ref":"changelog.html#v0-11-5"},{"type":"extras","title":"Fixes - Changelog","doc":"* fix parsing ranges with non-trivial integer bounds like `x..y` (Closes #119, h/t @maennchen)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.11.4 - Changelog","doc":"#","ref":"changelog.html#v0-11-4"},{"type":"extras","title":"Improvements - Changelog","doc":"Shoutout @milmazz for all the deprecation work below =)\n\n* Deprecations: Rewrite 1.16 Deprecations (h/t @milmazz for all the work here)\n  * add `//1` step to `Enum.slice/2|String.slice/2` with decreasing ranges\n  * `File.stream!(file, options, line_or_bytes)` => `File.stream!(file, line_or_bytes, options)`\n* Deprecations `Path.safe_relative_to/2` => `Path.safe_relative/2`","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.11.3 - Changelog","doc":"#","ref":"changelog.html#v0-11-3"},{"type":"extras","title":"Fixes - Changelog","doc":"* directives: fix infinite loop when encountering `@spec import(...) :: ...` (Closes #115, h/t @kerryb)\n* `with`: fix deletion of arrow-less `with` statements within function invocations","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.11.2 - Changelog","doc":"#","ref":"changelog.html#v0-11-2"},{"type":"extras","title":"Fixes - Changelog","doc":"* `pipes`: fix unpiping do-blocks into variables when the parent expression is a function invocation\n    like `a(if x do y end |> z(), b)` (Closes #114, h/t @wkirschbaum)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.11.1 - Changelog","doc":"#","ref":"changelog.html#v0-11-1"},{"type":"extras","title":"Fixes - Changelog","doc":"* `with`: fix `with` replacement when it's the only child of a `do` or `->` block (Closes #107, h/t @kerryb -- turns out those edge cases _did_ exist in the wild!)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.11.0 - Changelog","doc":"#","ref":"changelog.html#v0-11-0"},{"type":"extras","title":"Improvements - Changelog","doc":"##","ref":"changelog.html#improvements"},{"type":"extras","title":"Comments - Changelog","doc":"Styler will no longer make comments jump around in any situation, and will move comments with the appropriate node in all cases but module directive rearrangement (where they'll just be left behind - sorry! we're still working on it).\n\n* Keep comments in logical places when rewriting if/unless/cond/with (#79, #97, #101, #103)\n\n##","ref":"changelog.html#comments"},{"type":"extras","title":"With Statements - Changelog","doc":"This release has a slew of improvements for `with` statements. It's not surprising that there's lots of style rules for `with` given that just about any `case`, `if`, or even `cond do` could also be expressed as a `with`. They're very powerful! And with great power...\n\n* style trivial pattern matches ala `lhs <- rhs` to `lhs = rhs` (#86)\n* style `_ <- rhs` to `rhs`\n* style keyword `, do: ` to `do end` rather than wrapping multiple statements in parens\n* style statements all the way to `if` statements when appropriate (#100)\n\n##","ref":"changelog.html#with-statements"},{"type":"extras","title":"Other - Changelog","doc":"* Rewrite `{Map|Keyword}.merge(single_key: value)` to use `put/3` instead (#96)\n\n#","ref":"changelog.html#other"},{"type":"extras","title":"Fixes - Changelog","doc":"* `with`: various edge cases we can only hope no one's encountered and thus never reported","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.10.5 - Changelog","doc":"After being bitten by two of them in a row, Styler's test suite now makes sure that there are no\nidempotency bugs as part of its tests.\n\nIn short, we now have `assert style(x) == style(style(x))` as part of every test. Sorry for not thinking to include this before :)\n\n#","ref":"changelog.html#v0-10-5"},{"type":"extras","title":"Fixes - Changelog","doc":"* alias: fix single-module alias deletion newlines bug\n* comments: ensure all generated nodes always include line meta (#101)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.10.4 - Changelog","doc":"#","ref":"changelog.html#v0-10-4"},{"type":"extras","title":"Improvements - Changelog","doc":"* alias: delete noop single-module aliases (`alias Foo`, #87, h/t @mgieger)\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* pipes: unnest all pipe starts in one pass (`f(g(h(x))) |> j()` => `x |> h() |> g() |> f() |> j()`, #94, h/t @tomjschuster)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.10.3 - Changelog","doc":"#","ref":"changelog.html#v0-10-3"},{"type":"extras","title":"Improvements - Changelog","doc":"* charlists: leave charlist rewriting to elixir's formatter on elixir >= 1.15\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* charlists: rewrite empty charlist to use sigil (`''` => `~c\"\"`)\n* pipes: don't blow up extracting fully-qualified macros (`Foo.bar do end |> foo()`, #91, h/t @NikitaNaumenko)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.10.2 - Changelog","doc":"#","ref":"changelog.html#v0-10-2"},{"type":"extras","title":"Improvements - Changelog","doc":"* `with`: remove identity singleton else clause (eg `else {:error, e} -> {:error, e} end`, `else error -> error end`)","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.10.1 - Changelog","doc":"#","ref":"changelog.html#v0-10-1"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fix function head shrink-failures causing comments to jump into blocks (Closes #67, h/t @APB9785)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.10.0 - Changelog","doc":"#","ref":"changelog.html#v0-10-0"},{"type":"extras","title":"Improvements - Changelog","doc":"* hoist all block-starts to pipes to their own variables (makes styler play better with piped macros)\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* fix pipes starting with a macro do-block creating invalid ast (#83, h/t @mhanberg)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.9.7 - Changelog","doc":"#","ref":"changelog.html#v0-9-7"},{"type":"extras","title":"Fixes - Changelog","doc":"* rewrite pipes starting with `quote` blocks like we do with `case|if|cond|with` blocks (#82, h/t @SteffenDE)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.9.6 - Changelog","doc":"#","ref":"changelog.html#v0-9-6"},{"type":"extras","title":"Breaking Change - Changelog","doc":"* removed `mix style` task","ref":"changelog.html#breaking-change"},{"type":"extras","title":"v0.9.5 - Changelog","doc":"#","ref":"changelog.html#v0-9-5"},{"type":"extras","title":"Fixes - Changelog","doc":"* fix mistaking `Timex.now/1` in a pipe for `Timex.now/0` (#66, h/t @sabiwara)\n\n#","ref":"changelog.html#fixes"},{"type":"extras","title":"Removed style - Changelog","doc":"* stop rewriting `Timex.today/0` given that we allow `Timex.today/1` -- too inconsistent.","ref":"changelog.html#removed-style"},{"type":"extras","title":"v0.9.4 - Changelog","doc":"#","ref":"changelog.html#v0-9-4"},{"type":"extras","title":"Improvements - Changelog","doc":"* `if` statements: drop `else` clauses whose body is simply `nil`","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.9.3 - Changelog","doc":"#","ref":"changelog.html#v0-9-3"},{"type":"extras","title":"Fixes - Changelog","doc":"* fix `unless a do b else c end` rewrites to `if` not flopping do/else bodies! (#77, h/t @jcowgar)\n* fix pipes styling ranges with steps (`a..b//c`) incorrectly (#76, h/t @cschmatzler)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.9.2 - Changelog","doc":"#","ref":"changelog.html#v0-9-2"},{"type":"extras","title":"Fixes - Changelog","doc":"* fix exception styling module attributes named `@def` (we confused them with real `def`s, whoops!) (#75, h/t @randycoulman)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.9.1 - Changelog","doc":"the boolean blocks edition!\n\n#","ref":"changelog.html#v0-9-1"},{"type":"extras","title":"Improvements - Changelog","doc":"* auto-fix `Credo.Check.Refactor.CondStatements` (detects any truthy atom, not just `true`)\n* if/unless rewrites:\n  - `Credo.Check.Refactor.NegatedConditionsWithElse`\n  - `Credo.Check.Refactor.NegatedConditionsInUnless`\n  - `Credo.Check.Refactor.UnlessWithElse`","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.9.0 - Changelog","doc":"the with statement edition!\n\n#","ref":"changelog.html#v0-9-0"},{"type":"extras","title":"Improvements - Changelog","doc":"* Added right-hand-pattern-matching rewrites to `for` and `with` left arrow expressions `<-`\n  (ex: `with map = %{} <- foo()` => `with %{} = map <- foo`)\n* `with` statement rewrites, solving the following credo rules\n  * `Credo.Check.Readability.WithSingleClause`\n  * `Credo.Check.Refactor.RedundantWithClauseResult`\n  * `Credo.Check.Refactor.WithClauses`","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.8.5 - Changelog","doc":"#","ref":"changelog.html#v0-8-5"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fixed exception when encountering non-arrowed case statements ala `case foo, do: unquote(quoted)` (#69, h/t @brettinternet, nice)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.8.4 - Changelog","doc":"#","ref":"changelog.html#v0-8-4"},{"type":"extras","title":"Fixes - Changelog","doc":"* Timex related fixes (#66):\n  * Rewrite `Timex.now/1` to `DateTime.now!/1` instead of `DateTime.utc_now/1`\n  * Only rewrite `Timex.today/0`, don't change `Timex.today/1`","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.8.3 - Changelog","doc":"#","ref":"changelog.html#v0-8-3"},{"type":"extras","title":"Improvements - Changelog","doc":"* DateTime rewrites (#62, ht @milmazz)\n  * `DateTime.compare` => `DateTime.{before/after}` (elixir >= 1.15)\n  * `Timex.now` => `DateTime.utc_now`\n  * `Timex.today` => `Date.utc_today`\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* Pipes: add  `!=`, `!==`, `===`, `and`, and `or` to list of valid infix operators (#64)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.8.2 - Changelog","doc":"#","ref":"changelog.html#v0-8-2"},{"type":"extras","title":"Fixes - Changelog","doc":"* Pipes always de-sugars keyword lists when unpiping them (#60)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.8.1 - Changelog","doc":"#","ref":"changelog.html#v0-8-1"},{"type":"extras","title":"Fixes - Changelog","doc":"* ModuleDirectives doesn't mistake variables for directives (#57, h/t @leandrocp)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.8.0 - Changelog","doc":"#","ref":"changelog.html#v0-8-0"},{"type":"extras","title":"Improvements (Bug Fix!?) - Changelog","doc":"* ModuleDirectives no longer throws comments around a file when hoisting directives up (#53)","ref":"changelog.html#improvements-bug-fix"},{"type":"extras","title":"v0.7.14 - Changelog","doc":"#","ref":"changelog.html#v0-7-14"},{"type":"extras","title":"Improvements - Changelog","doc":"* rewrite `Logger.warn/1,2` to `Logger.warning/1,2` due to Elixir 1.15 deprecation","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.7.13 - Changelog","doc":"#","ref":"changelog.html#v0-7-13"},{"type":"extras","title":"Fixes - Changelog","doc":"* don't unpipe single-piped `unquote` expressions (h/t @elliottneilclark)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.7.12 - Changelog","doc":"#","ref":"changelog.html#v0-7-12"},{"type":"extras","title":"Fixes - Changelog","doc":"* fix 0-arity paren removal on metaprogramming creating uncompilable code (h/t @simonprev)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.7.11 - Changelog","doc":"#","ref":"changelog.html#v0-7-11"},{"type":"extras","title":"Fixes - Changelog","doc":"* fix crash from `mix style` running plugins as part of formatting (no longer runs formatter plugins)\n\n#","ref":"changelog.html#fixes"},{"type":"extras","title":"Improvements - Changelog","doc":"* single-quote charlists are rewritten to use the `~c` sigil (`'foo'` -> `~c'foo'`) (h/t @fhunleth)\n* `mix style` warns the user that Styler is primarily meant to be used as a plugin","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.7.10 - Changelog","doc":"#","ref":"changelog.html#v0-7-10"},{"type":"extras","title":"Fixes - Changelog","doc":"* fix crash when encountering single-quote charlists (h/t @fhunleth)\n\n#","ref":"changelog.html#fixes"},{"type":"extras","title":"Improvements - Changelog","doc":"* single-quote charlists are rewritten to use the `~c` sigil (`'foo'` -> `~c'foo'`)\n* when encountering `_ = bar ->`, replace it with `bar ->`","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.7.9 - Changelog","doc":"#","ref":"changelog.html#v0-7-9"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fix a toggle state resulting from (ahem, nonsense) code like `_ = bar ->` encountering ParameterPatternMatching style","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.7.8 - Changelog","doc":"#","ref":"changelog.html#v0-7-8"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fix crash trying to remove 0-arity parens from metaprogramming ala `def unquote(foo)()`","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.7.7 - Changelog","doc":"#","ref":"changelog.html#v0-7-7"},{"type":"extras","title":"Improvements - Changelog","doc":"* Rewrite `Enum.into/2,3` into `Map.new/1,2` when the collectable is `%{}` or `Map.new/0`","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.7.6 - Changelog","doc":"#","ref":"changelog.html#v0-7-6"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fix crash when single pipe had inner defs (h/t [@michallepicki](https://github.com/adobe/elixir-styler/issues/39))","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.7.5 - Changelog","doc":"#","ref":"changelog.html#v0-7-5"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fix bug from `ParameterPatternMatching` implementation that re-ordered pattern matching in `cond do` `->` clauses","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.7.4 - Changelog","doc":"#","ref":"changelog.html#v0-7-4"},{"type":"extras","title":"Features - Changelog","doc":"* Implement `Credo.Check.Readability.PreferImplicitTry`\n* Implement `Credo.Check.Consistency.ParameterPatternMatching` for `def|defp|fn|case`","ref":"changelog.html#features"},{"type":"extras","title":"v0.7.3 - Changelog","doc":"#","ref":"changelog.html#v0-7-3"},{"type":"extras","title":"Features - Changelog","doc":"* Remove parens from 0-arity function definitions (`Credo.Check.Readability.ParenthesesOnZeroArityDefs`)","ref":"changelog.html#features"},{"type":"extras","title":"v0.7.2 - Changelog","doc":"#","ref":"changelog.html#v0-7-2"},{"type":"extras","title":"Features - Changelog","doc":"* Rewrite `case ... true -> ...; _ -> ...` to `if` statements as well","ref":"changelog.html#features"},{"type":"extras","title":"v0.7.1 - Changelog","doc":"#","ref":"changelog.html#v0-7-1"},{"type":"extras","title":"Features - Changelog","doc":"* Rewrite `case ... true / else ->` to be `if` statements","ref":"changelog.html#features"},{"type":"extras","title":"v0.7.0 - Changelog","doc":"#","ref":"changelog.html#v0-7-0"},{"type":"extras","title":"Features - Changelog","doc":"* `Styler.Style.Simple`:\n    * Optimize `Enum.reverse(foo) ++ bar` to `Enum.reverse(foo, bar)`\n* `Styler.Style.Pipes`:\n    * Rewrite `|> (& ...).()` to `|> then(& ...)` (`Credo.Check.Readability.PipeIntoAnonymousFunctions`)\n    * Add parens to 1-arity pipe functions (`Credo.Check.Readability.OneArityFunctionInPipe`)\n    * Optimize `a |> Enum.reverse() |> Enum.concat(enum)` to `Enum.reverse(a, enum)`","ref":"changelog.html#features"},{"type":"extras","title":"v0.6.1 - Changelog","doc":"#","ref":"changelog.html#v0-6-1"},{"type":"extras","title":"Improvements - Changelog","doc":"* Better error handling: `mix format` will still format files if a style fails\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* `mix style`: only run on `.ex` and `.exs` files\n* `ModuleDirectives`: now expands `alias __MODULE__.{A, B}` (h/t [@adriankumpf](https://github.com/adriankumpf))","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.6.0 - Changelog","doc":"#","ref":"changelog.html#v0-6-0"},{"type":"extras","title":"Features - Changelog","doc":"* `mix style`: brought back to life for folks who want to incrementally introduce Styler\n\n#","ref":"changelog.html#features"},{"type":"extras","title":"Fixes - Changelog","doc":"* `Styler.Style.Pipes`:\n   * include `x in y` and `^foo` (for ecto) as a valid pipe starts\n   * work even harder to keep rewrites on one line","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.5.2 - Changelog","doc":"#","ref":"changelog.html#v0-5-2"},{"type":"extras","title":"Fixes - Changelog","doc":"* `ModuleDirectives`: handle dynamic module names\n* `Pipes`: include `Ecto.Query.from` and `Query.from` as valid pipe starts","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.5.1 - Changelog","doc":"#","ref":"changelog.html#v0-5-1"},{"type":"extras","title":"Improvements - Changelog","doc":"* Sped up styling just a little bit","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.5.0 - Changelog","doc":"#","ref":"changelog.html#v0-5-0"},{"type":"extras","title":"Improvements - Changelog","doc":"* `Styler` now implements `Mix.Task.Format`, meaning it is now an Elixir formatter plugin.\nSee the README for new installation & usage instructions\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Breaking Change! Wooo! - Changelog","doc":"* the `mix style` task has been removed","ref":"changelog.html#breaking-change-wooo"},{"type":"extras","title":"v0.4.1 - Changelog","doc":"#","ref":"changelog.html#v0-4-1"},{"type":"extras","title":"Improvements - Changelog","doc":"* `Pipes` rewrites `|> Enum.into(%{}[, mapper])` and `Enum.into(Map.new()[, mapper])` to `Map.new/1,2` calls","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.4.0 - Changelog","doc":"#","ref":"changelog.html#v0-4-0"},{"type":"extras","title":"Improvements - Changelog","doc":"* `Pipes` rewrites some two-step processes into one, fixing these credo issues in pipe chains:\n    * `Credo.Check.Refactor.FilterCount`\n    * `Credo.Check.Refactor.MapJoin`\n    * `Credo.Check.Refactor.MapInto`\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* `ModuleDirectives` handles even weirder places to hide your aliases (anonymous functions, in this case)\n* `Pipes` tries even harder to keep single-pipe rewrites of invocations on one line","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.3.1 - Changelog","doc":"#","ref":"changelog.html#v0-3-1"},{"type":"extras","title":"Fixes - Changelog","doc":"* `Pipes`\n    * fixed omission of `==` as a valid pipe start operator (h/t @peake100 for the issue)\n    * fixed rewrite of `a |> b`, where `b` was invoked without parenthesis","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.3.0 - Changelog","doc":"#","ref":"changelog.html#v0-3-0"},{"type":"extras","title":"Improvements - Changelog","doc":"* Enabled `Defs` style and overhauled it to properly handles comments\n* Optimized and tweaked `ModuleDirectives` style\n    * Now culls newlines between \"groups\" of the same directive\n    * sorts `@behaviour` directives\n    * orders directives within non defmodule contexts (eg, a `def do`) if there's at least one `alias|require|use|import`\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* `Pipes` will try to keep single-pipe rewrites on one line","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.2.0 - Changelog","doc":"#","ref":"changelog.html#v0-2-0"},{"type":"extras","title":"Improvements - Changelog","doc":"* Added `ModuleDirectives` style\n    * Note that this is potentially destructive in some rare cases. See moduledoc for more.\n    * This supersedes the `Aliases` style, which has been removed.\n* `mix style -` reads and writes to stdin/stdout\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* `Pipes` style is now aware of `unless` blocks","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.1.1 - Changelog","doc":"#","ref":"changelog.html#v0-1-1"},{"type":"extras","title":"Improvements - Changelog","doc":"* Lots of README tweaking =)\n* Optimized some Zipper operations\n* Added `Simple` style, replacing the following Credo rule:\n    * `Credo.Check.Readability.LargeNumbers`\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* Exceptions while parsing code now appropriately render filename rather than `nofile:xx`\n* Fixed opaque `Zipper.path()` typespec implementation mismatches (thanks @sega-yarkin)\n* Made `ex_doc` dev only, removing it as a dependency for users of Styler","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.1.0 - Changelog","doc":"#","ref":"changelog.html#v0-1-0"},{"type":"extras","title":"Improvements - Changelog","doc":"* Initial release of Styler\n* Added `Aliases` style, replacing the following Credo rules:\n    * `Credo.Check.Readability.AliasOrder`\n    * `Credo.Check.Readability.MultiAlias`\n    * `Credo.Check.Readability.UnnecessaryAliasExpansion`\n* Added `Pipes` style, replacing the following Credo rules:\n    * `Credo.Check.Readability.BlockPipe`\n    * `Credo.Check.Readability.SinglePipe`\n    * `Credo.Check.Refactor.PipeChainStart`\n* Added `Defs` style (currently disabled by default)","ref":"changelog.html#improvements"},{"type":"extras","title":"Styler","doc":"# Styler\n\nStyler is an Elixir formatter plugin that's combination of `mix format` and `mix credo`, except instead of telling\nyou what's wrong, it just rewrites the code for you to fit its style rules.\n\nYou can learn more about the history, purpose and implementation of Styler from our talk: [Styler: Elixir Style-Guide Enforcer @ GigCity Elixir 2023](https://www.youtube.com/watch?v=6pF8Hl5EuD4)","ref":"readme.html"},{"type":"extras","title":"Features - Styler","doc":"- auto-fixes [many credo rules](docs/credo.md), meaning you can turn them off to speed credo up\n- [keeps a strict module layout](docs/module_directives.md#directive-organization)\n- alphabetizes module directives\n- [extracts repeated aliases](docs/module_directives.md#alias-lifting)\n- pipes and unpipes function calls based on the number of calls\n- optimizes standard library calls (`a |> Enum.map(m) |> Enum.into(Map.new)` => `Map.new(a, m)`)\n- replaces strings with sigils when the string has many escaped quotes\n- ... and so much more\n\n[See our Rewrites documentation on hexdocs for all the nitty-gritty on what all Styler does](https://hexdocs.pm/styler/)","ref":"readme.html#features"},{"type":"extras","title":"Who is Styler for? - Styler","doc":"Styler was designed for a large team (40+ engineers) working in a single codebase. It helps remove fiddly code review comments and removes failed linter CI slowdowns, helping teams get things done faster. Teams in similar situations might appreciate Styler.\n\nIts automations are also extremely valuable for taming legacy elixir codebases or just refactoring in general. Some of its rewrites have inspired code actions in elixir language servers.\n\nConversely, Styler probably _isn't_ a good match for:\n\n- libraries\n- experimental, macro-heavy codebases\n- small teams that don't want to think about code standards","ref":"readme.html#who-is-styler-for"},{"type":"extras","title":"Installation - Styler","doc":"Add `:styler` as a dependency to your project's `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:styler, \"~> 1.0.0-rc.1\", only: [:dev, :test], runtime: false},\n  ]\nend\n```\n\nThen add `Styler` as a plugin to your `.formatter.exs` file\n\n```elixir\n[\n  plugins: [Styler]\n]\n```\n\nAnd that's it! Now when you run `mix format` you'll also get the benefits of Styler's Stylish Stylings.\n\n**Speed**: Expect the first run to take some time as `Styler` rewrites violations of styles and bottlenecks on disk I/O. Subsequent formats formats won't take noticeably more time.\n\n#","ref":"readme.html#installation"},{"type":"extras","title":"Configuration - Styler","doc":"Styler can be configured in your `.formatter.exs` file\n\n```elixir\n[\n  plugins: [Styler],\n  styler: [\n    alias_lifting_exclude: [...]\n  ]\n]\n```\n\nStyler's only current configuration option is `:alias_lifting_exclude`, which accepts a list of atoms to _not_ lift. See the [Module Directive documentation](docs/module_directives.md#alias-lifting) for more.\n\n##","ref":"readme.html#configuration"},{"type":"extras","title":"No Credo-Style Enable/Disable - Styler","doc":"Styler [will not add configuration](https://github.com/adobe/elixir-styler/pull/127#issuecomment-1912242143) for ad-hoc enabling/disabling of rewrites. Sorry! Its implementation simply does not support that approach. There are however many forks out there that have attempted this; please explore the [Github forks tab](https://github.com/adobe/elixir-styler/forks) to see if there's a project that suits your needs or that you can draw inspiration from.\n\nUltimately Styler is @adobe's internal tool that we're happy to share with the world. We're delighted if you like it as is, and just as excited if it's a starting point for you to make something even better for yourself.\n\n## !Styler can change the behaviour of your program!\n\nThe best example of the way in which Styler changes the meaning of your code is the following rewrite:\n```elixir\n# Before: this case statement...\ncase foo do\n  true -> :ok\n  false -> :error\nend\n\n# After: ... is rewritten by Styler to be an if statement!.\nif foo do\n  :ok\nelse\n  :error\nend\n```\n\nThese programs are not semantically equivalent. The former would raise if `foo` returned any value other than `true` or `false`, while the latter blissfully completes.\n\nHowever, Styler is about _style_, and the `if` statement is (in our opinion) of much better style. If the exception behaviour was intentional on the code author's part, they should have written the program like this:\n\n```elixir\ncase foo do\n  true -> :ok\n  false -> :error\n  other -> raise \"expected `true` or `false`, got: #{inspect other}\"\nend\n```\n\nAlso good style! But Styler assumes that most of the time people just meant the `if` equivalent of the code, and so makes that change. If issues like this bother you, Styler probably isn't the tool you're looking for.","ref":"readme.html#no-credo-style-enable-disable"},{"type":"extras","title":"Thanks & Inspiration - Styler","doc":"### [Sourceror](https://github.com/doorgan/sourceror/)\n\nStyler's first incarnation was as one-off scripts to rewrite an internal codebase to allow Credo rules to be turned on.\n\nThese rewrites were entirely powered by the terrific `Sourceror` library.\n\nWhile `Styler` no longer relies on `Sourceror`, we're grateful for its author's help with those scripts, the inspiration\nSourceror provided in showing us what was possible, and the changes to the Elixir AST APIs that it drove.\n\nStyler's [AST-Zipper](`m:Styler.Zipper`) implementation in this project was forked from Sourceror. Zipper has been a crucial\npart of our ability to ergonomically zip around (heh) Elixir AST.\n\n### [Credo](https://github.com/rrrene/credo/)\n\nWe never would've bothered trying to rewrite our codebase if we didn't have Credo rules we wanted to apply.\n\nCredo's tests and implementations were referenced for implementing Styles that took the work the rest of the way.\n\nThanks to Credo & the Elixir community at large for coalescing around many of these Elixir style credos.","ref":"readme.html#thanks-inspiration"},{"type":"extras","title":"Basic Styles","doc":"# Simple (Single Node) Styles\n\nFunction Performance & Readability Optimizations\n\nOptimizing for either performance or readability, probably both!\nThese apply to the piped versions as well","ref":"styles.html"},{"type":"extras","title":"Strings to Sigils - Basic Styles","doc":"Rewrites strings with 4 or more escaped quotes to string sigils with an alternative delimiter.\nThe delimiter will be one of `\" ( { | [ ' < /`, chosen by which would require the fewest escapes, and otherwise preferred in the order listed.\n\n* `\"{\\\"errors\\\":[\\\"Not Authorized\\\"]}\"` => `~s({\"errors\":[\"Not Authorized\"]})`","ref":"styles.html#strings-to-sigils"},{"type":"extras","title":"Large Base 10 Numbers - Basic Styles","doc":"Style base 10 numbers with 5 or more digits to have a `_` every three digits.\nFormatter already does this except it doesn't rewrite \"typos\" like `100_000_0`.\n\nIf you're concerned that this breaks your team's formatting for things like \"cents\" (like \"$100\" being written as `100_00`),\nconsider using a library made for denoting currencies rather than raw elixir integers.\n\n| Before | After |\n|--------|-------|\n| `10000 ` | `10_000`|\n| `1_0_0_0_0` | `10_000` (elixir's formatter leaves the former as-is)|\n| `-543213 ` | `-543_213`|\n| `123456789 ` | `123_456_789`|\n| `55333.22 ` | `55_333.22`|\n| `-123456728.0001 ` | `-123_456_728.0001`|\n\n## `Enum.into` -> `X.new`\n\nThis rewrite is applied when the collectable is a new map, keyword list, or mapset via `Enum.into/2,3`.\n\nThis is an improvement for the reader, who gets a more natural language expression: \"make a new map from enum\" vs \"enumerate enum and collect its elements into a new map\"\n\nNote that all of the examples below also apply to pipes (`enum |> Enum.into(...)`)\n\n| Before | After |\n|--------|-------|\n| `Enum.into(enum, %{})` | `Map.new(enum)`|\n| `Enum.into(enum, Map.new())` | `Map.new(enum)`|\n| `Enum.into(enum, Keyword.new())` | `Keyword.new(enum)`|\n| `Enum.into(enum, MapSet.new())` | `Keyword.new(enum)`|\n| `Enum.into(enum, %{}, fn x -> {x, x} end)` | `Map.new(enum, fn x -> {x, x} end)`|\n| `Enum.into(enum, [])` | `Enum.to_list(enum)` |\n| `Enum.into(enum, [], mapper)` | `Enum.map(enum, mapper)`|","ref":"styles.html#large-base-10-numbers"},{"type":"extras","title":"Map/Keyword.merge w/ single key literal -> X.put - Basic Styles","doc":"`Keyword.merge` and `Map.merge` called with a literal map or keyword argument with a single key are rewritten to the equivalent `put`, a cognitively simpler function.\n\n| Before | After |\n|--------|-------|\n| `Keyword.merge(kw, [key: :value])` | `Keyword.put(kw, :key, :value)` |\n| `Map.merge(map, %{key: :value})` | `Map.put(map, :key, :value)` |\n| `Map.merge(map, %{key => value})` | `Map.put(map, key, value)` |\n| `map \\|> Map.merge(%{key: value}) \\|> foo()` | `map \\|> Map.put(:key, value) \\|> foo()` |","ref":"styles.html#map-keyword-merge-w-single-key-literal-x-put"},{"type":"extras","title":"Map/Keyword.drop w/ single key -> X.delete - Basic Styles","doc":"In the same vein as the `merge` style above, `[Map|Keyword].drop/2` with a single key to drop are rewritten to use `delete/2`\n| Before | After |\n|--------|-------|\n| `Map.drop(map, [key])` | `Map.delete(map, key)`|\n| `Keyword.drop(kw, [key])` | `Keyword.delete(kw, key)`|\n\n## `Enum.reverse/1` and concatenation -> `Enum.reverse/2`\n\n`Enum.reverse/2` optimizes a two-step reverse and concatenation into a single step.\n\n| Before | After |\n|--------|-------|\n| `Enum.reverse(foo) ++ bar` | `Enum.reverse(foo, bar)`|\n| `baz \\|> Enum.reverse() \\|> Enum.concat(bop)` | `Enum.reverse(baz, bop)`|\n\n## `Timex.now/0` ->` DateTime.utc_now/0`\n\nTimex certainly has its uses, but knowing what stdlib date/time struct is returned by `now/0` is a bit difficult!\n\nWe prefer calling the actual function rather than its rename in Timex, helping the reader by being more explicit.\n\nThis also hews to our internal styleguide's \"Don't make one-line helper functions\" guidance.\n\n## `DateModule.compare/2` -> `DateModule.[before?|after?]`\n\nAgain, the goal is readability and maintainability. `before?/2` and `after?/2` were implemented long after `compare/2`,\nso it's not unusual that a codebase needs a lot of refactoring to be brought up to date with these new functions.\nThat's where Styler comes in!\n\nThe examples below use `DateTime.compare/2`, but the same is also done for `NaiveDateTime|Time|Date.compare/2`\n\n| Before | After |\n|--------|-------|\n| `DateTime.compare(start, end_date) == :gt` | `DateTime.after?(start, end_date)` |\n| `DateTime.compare(start, end_date) == :lt` | `DateTime.before?(start, end_date)` |","ref":"styles.html#map-keyword-drop-w-single-key-x-delete"},{"type":"extras","title":"Implicit Try - Basic Styles","doc":"Styler will rewrite functions whose entire body is a try/do to instead use the implicit try syntax, per Credo's `Credo.Check.Readability.PreferImplicitTry`\n\nThe following example illustrates the most complex case, but Styler happily handles just basic try do/rescue bodies just as easily.\n\n#","ref":"styles.html#implicit-try"},{"type":"extras","title":"Before - Basic Styles","doc":"```elixir\ndef foo() do\n  try do\n    uh_oh()\n  rescue\n    exception -> {:error, exception}\n  catch\n    :a_throw -> {:error, :threw!}\n  else\n    try_has_an_else_clause? -> {:did_you_know, try_has_an_else_clause?}\n  after\n    :done\n  end\nend\n```\n\n#","ref":"styles.html#before"},{"type":"extras","title":"After - Basic Styles","doc":"```elixir\ndef foo() do\n  uh_oh()\nrescue\n  exception -> {:error, exception}\ncatch\n  :a_throw -> {:error, :threw!}\nelse\n  try_has_an_else_clause? -> {:did_you_know, try_has_an_else_clause?}\nafter\n  :done\nend\n```","ref":"styles.html#after"},{"type":"extras","title":"Remove parenthesis from 0-arity function & macro definitions - Basic Styles","doc":"The author of the library disagrees with this style convention :) BUT, the wonderful thing about Styler is it lets you write code how _you_ want to, while normalizing it for reading for your entire team. The most important thing is not having to think about the style, and instead focus on what you're trying to achieve.\n\n| Before | After |\n|--------|-------|\n| `def foo()` | `def foo`|\n| `defp foo()` | `defp foo`|\n| `defmacro foo()` | `defmacro foo`|\n| `defmacrop foo()` | `defmacrop foo`|","ref":"styles.html#remove-parenthesis-from-0-arity-function-macro-definitions"},{"type":"extras","title":"Elixir Deprecation Rewrites - Basic Styles","doc":"#","ref":"styles.html#elixir-deprecation-rewrites"},{"type":"extras","title":"1.15+ - Basic Styles","doc":"| Before | After |\n|--------|-------|\n| `Logger.warn` | `Logger.warning`|\n| `Path.safe_relative_to/2` | `Path.safe_relative/2`|\n| `~R/my_regex/` | `~r/my_regex/`|\n| `Enum/String.slice/2` with decreasing ranges | add explicit steps to the range * |\n| `Date.range/2` with decreasing range | `Date.range/3` *|\n| `IO.read/bin_read` with `:all` option | replace `:all` with `:eof`|\n\n\\* For both of the \"decreasing range\" changes, the rewrite can only be applied if the range is being passed as an argument to the function.\n\n#","ref":"styles.html#1-15"},{"type":"extras","title":"1.16+ - Basic Styles","doc":"| Before | After |\n|--------|-------|\n|`File.stream!(file, options, line_or_bytes)` | `File.stream!(file, line_or_bytes, options)`|","ref":"styles.html#1-16"},{"type":"extras","title":"Code Readability - Basic Styles","doc":"- put matches on right\n- `Credo.Check.Readability.PreferImplicitTry`","ref":"styles.html#code-readability"},{"type":"extras","title":"Function Definitions - Basic Styles","doc":"- Shrink multi-line function defs\n- Put assignments on the right","ref":"styles.html#function-definitions"},{"type":"extras","title":"Pipe Chains","doc":"# Pipe Chains","ref":"pipes.html"},{"type":"extras","title":"Pipe Start - Pipe Chains","doc":"- raw value\n- blocks are extracted to variables\n- ecto's `from` is allowed","ref":"pipes.html#pipe-start"},{"type":"extras","title":"Piped function rewrites - Pipe Chains","doc":"- add parens to function calls `|> fun |>` => `|> fun() |>`\n- remove unnecessary `then/2`: `|> then(&f(&1, ...))` -> `|> f(...)`\n- add `then` when defining anon funs in pipe `|> (& &1).() |>` => `|> |> then(& &1) |>`","ref":"pipes.html#piped-function-rewrites"},{"type":"extras","title":"Piped function optimizations - Pipe Chains","doc":"Two function calls into one! Tries to fit everything on one line when shrinking.\n\n| Before | After |\n|--------|-------|\n| `lhs \\|> Enum.reverse() \\|> Enum.concat(enum)` | `lhs \\|> Enum.reverse(enum)` (also Kernel.++) |\n| `lhs \\|> Enum.filter(filterer) \\|> Enum.count()` | `lhs \\|> Enum.count(count)` |\n| `lhs \\|> Enum.map(mapper) \\|> Enum.join(joiner)` | `lhs \\|> Enum.map_join(joiner, mapper)` |\n| `lhs \\|> Enum.map(mapper) \\|> Enum.into(empty_map)` | `lhs \\|> Map.new(mapper)` |\n| `lhs \\|> Enum.map(mapper) \\|> Map.new()` | `lhs \\|> Map.new(mapper)` mapset & keyword also |","ref":"pipes.html#piped-function-optimizations"},{"type":"extras","title":"Unpiping Single Pipes - Pipe Chains","doc":"- notably, optimizations might turn a 2 pipe into a single pipe\n- doesn't unpipe when we're starting w/ quote\n- pretty straight forward i daresay","ref":"pipes.html#unpiping-single-pipes"},{"type":"extras","title":"Control Flow Macros (if, case, ...)","doc":"# Control Flow Macros (`case`, `if`, `unless`, `cond`, `with`)\n\nElixir's Kernel documentation refers to these structures as \"macros for control-flow\".\nWe often refer to them as \"blocks\" in our changelog, which is a much worse name, to be sure.\n\nYou're likely here just to see what Styler does, in which case, please [click here to skip](#if-and-unless) the following manifesto on our philosophy regarding the usage of these macros.","ref":"control_flow_macros.html"},{"type":"extras","title":"Which Control Flow Macro Should I Use? - Control Flow Macros (if, case, ...)","doc":"The number of \"blocks\" in Elixir means there are many ways to write semantically equivalent code, often leaving developers [in the dark as to which structure they should use.](https://www.reddit.com/r/elixir/comments/1ctbtcl/i_am_completely_lost_when_it_comes_to_which/)\n\nWe believe readability is enhanced by using the simplest api possible, whether we're talking about internal module function calls or standard-library macros.\n\n#","ref":"control_flow_macros.html#which-control-flow-macro-should-i-use"},{"type":"extras","title":"use `case`, `if`, or `unless` when... - Control Flow Macros (if, case, ...)","doc":"We advocate for `case` and `if` as the first tools to be considered for any control flow as they are the two simplest blocks. If a branch _can_ be expressed with an `if` statement, it _should_ be. Otherwise, `case` is the next best choice. In situations where developers might reach for an `if/elseif/else` block in other languages, `cond do` should be used.\n\n(`cond do` seems to see a paucity of use in the language, but many complex nested expressions or with statements can be improved by replacing them with a `cond do`).\n\n`unless` is a special case of `if` meant to make code read as natural-language (citation needed). While it sometimes succeeds in this goal, its absence in most programming languages often makes it feel cumbersome to programmers with non-Ruby backgrounds. Thankfully, with Styler's help developers don't need to ever reach for `unless` - expressions that are \"simpler\" with its use are automatically rewritten to use it.\n\n#","ref":"control_flow_macros.html#use-case-if-or-unless-when"},{"type":"extras","title":"use `with` when... - Control Flow Macros (if, case, ...)","doc":"> `with` great power comes great responsibility\n>\n> - Uncle Ben\n\n\nAs the most powerful of the Kernel control-flow expressions, `with` requires the most cognitive overhead to understand. Its power means that we can use it as a replacement for anything we might express using a `case`, `if`, or `cond` (especially with the liberal application of small private helper functions).\n\nUnfortunately, this has lead to a proliferation of `with` in codebases where simpler expressions would have sufficed, meaning a lot of Elixir code ends up being harder for readers to understand than it needs to be.\n\nThus, `with` is the control-flow structure of last resort. We advocate that `with` **should only be used when more basic expressions do not suffice or become overly verbose**. As for verbosity, we subscribe to the [Chris Keathley school of thought](https://www.youtube.com/watch?v=l-8ghbdRB1w) that judicious nesting of control flow blocks within a function isn't evil and more-often-than-not is superior to spreading implementation over many small single-use functions. We'd even go so far as to suggest that cyclomatic complexity is an inexact measure of code quality, with more than a few false negatives and many false positives.\n\n`with` is a great way to unnest multiple `case` statements when every failure branch of those statements results in the same error. This is easily and succinctly expressed with `with`'s `else` block: `else (_ -> :error)`. As Keathley says though, [Avoid Else In With Blocks](https://keathley.io/blog/good-and-bad-elixir.html#avoid-else-in-with-blocks). Having multiple else clauses \"means that the error conditions matter. Which means that you don’t want `with` at all. You want `case`.\"\n\nIt's acceptable to use one-line `with` statements (eg `with {:ok, _} <- Repo.update(changeset), do: :ok`) to signify that other branches are uninteresting or unmodified by your code, but ultimately that can hide the possible returns of a function from the reader, making it more onerous to debug all possible branches of the code in their mental model of the function. In other words, ideally all function calls in a `with` statement head have obvious error types for the reader, leaving their omission in the code acceptable as the reader feels no need to investigate further. The example at the start of this paragraph with an `Ecto.Repo` call is a good example, as most developers in a codebase using Ecto are expected to be familiar with its basic API.\n\nUsing `case` rather than `with` for branches with unusual failure types can help document code as well as save the reader time in tracking down types. For example, replacing the following with a `with` statement that only matched against the `{:ok, _}` tuple would hide from readers that an atypically-shaped 3-tuple is returned when things go wrong.\n\n```elixir\ncase some_http_call() do\n  {:ok, _response} -> :ok\n  {:error, http_error, response} -> {:error, http_error, response}\nend\n```\n\n## `if` and `unless`\n\nStyler removes `else: nil` clauses:\n\n```elixir\nif a, do: b, else: nil\n# styled:\nif a, do: b\n```\n\n#","ref":"control_flow_macros.html#use-with-when"},{"type":"extras","title":"Negation Inversion - Control Flow Macros (if, case, ...)","doc":"Styler removes negators in the head of `if` and `unless` statements by \"inverting\" the statement.\nThe following operators are considered \"negators\": `!`, `not`, `!=`, `!==`\n\n\nExamples:\n\n```elixir\n# negated `if` statement with no `else` clause are rewritten to `unless`\nif not x, do: y\n# Styled:\nunless x, do: y\n\n# negated `if` statements with an `else` clause have their clauses inverted and negation removed\nif !x, do: y, else: z\n# Styled:\nif x, do: z, else: y\n\n# negated `unless` statements are rewritten to `if`\nunless x != y, do: z\n# B styled:\nif x == y, do: z\n\n# `unless` with `else` is verboten; these are always rewritten to `if` statements\nunless x, do: y, else: z\n# styled:\nif x, do: z, else: y\n```\n\nBecause elixir relies on truthy/falsey values for its `if` statements, boolean casting is unnecessary and so double negation is simply removed.\n\n```elixir\nif !!x, do: y\n# styled:\nif x, do: y\n```\n\n## `case`\n\n### \"Erlang heritage\" `case` true/false -> `if`\n\nTrivial true/false `case` statements are rewritten to `if` statements. While this results in a [semantically different program](https://github.com/rrrene/credo/issues/564#issue-338349517), we argue that it results in a better program for maintainability. If the developer wants a their case statement to raise when receiving a non-boolean value as a feature of the program, they would better serve their callers by raising something more descriptive.\n\nIn other words, Styler leaves the code with better style, trumping obscure exception design :)\n\n```elixir\n# instead of this\ncase foo do\n  true -> :ok\n  false -> :error\nend\n\n# do this\nif foo do\n  :ok\nelse\n  :error\nend\n\n# OR this. readers now know that the exception is an intentional design,\n# rather than an accidental \"feature\"\ncase foo do\n  true -> :ok\n  false -> :error\n  other -> raise \"expected `true` or `false`, got: #{inspect other}\"\nend\n```\n\n## `cond`\n\nStyler has only one `cond` statement rewrite: replace 2-clause statements with `if` statements.\n\n```elixir\n# Given\ncond do\n  a -> b\n  true -> c\nend\n# Styled\nif a do\n  b\nelse\n  c\nend\n```\n\n## `with`\n\n`with` statements are extremely expressive. Styler tries to remove any unnecessary complexity from them in the following ways.\n\n#","ref":"control_flow_macros.html#negation-inversion"},{"type":"extras","title":"Remove Identity Else Clause - Control Flow Macros (if, case, ...)","doc":"Like if statements with `nil` as their else clause, the identity `else` clause is the default for `with` statements and so is removed.\n\n```elixir\n# Given\nwith :ok <- b(), :ok <- b() do\n  foo()\nelse\n  error -> error\nend\n# Styled:\nwith :ok <- b(), :ok <- b() do\n  foo()\nend\n```\n\n#","ref":"control_flow_macros.html#remove-identity-else-clause"},{"type":"extras","title":"Remove The Statement Entirely - Control Flow Macros (if, case, ...)","doc":"While you might think \"surely this kind of code never appears in the wild\", it absolutely does. Typically it's the result of someone refactoring a pattern away and not looking at the larger picture and realizing that the with statement now serves no purpose.\n\nMaybe someday the compiler will warn about these use cases. Until then, Styler to the rescue.\n\n```elixir\n# Given:\nwith a <- b(),\n     c <- d(),\n     e <- f(),\n     do: g,\n     else: (_ -> h)\n# Styled:\na = b()\nc = d()\ne = f()\ng\n\n# Given\nwith value <- arg do\n  value\nend\n# Styled:\narg\n```\n\n#","ref":"control_flow_macros.html#remove-the-statement-entirely"},{"type":"extras","title":"Replace `_ <- rhs` with `rhs` - Control Flow Macros (if, case, ...)","doc":"This is another case of \"less is more\" for the reader.\n\n```elixir\n# Given\nwith :ok <- x,\n     _ <- y(),\n     {:ok, _} <- z do\n  :ok\nend\n# Styled:\nwith :ok <- x,\n     y(),\n     {:ok, _} <- z do\n  :ok\nend\n```\n\n#","ref":"control_flow_macros.html#replace-_-rhs-with-rhs"},{"type":"extras","title":"Replace non-branching `bar <-` with `bar =` - Control Flow Macros (if, case, ...)","doc":"`<-` is for branching. If the lefthand side is the trivial match (a bare variable), Styler rewrites it to use the `=` operator instead.\n\n```elixir\n# Given\nwith :ok <- foo(),\n     bar <- baz(),\n     :ok <- woo(),\n     do: {:ok, bar}\n# Styled\n with :ok <- foo(),\n      bar = baz(),\n      :ok <- woo(),\n      do: {:ok, bar}\n```\n\n#","ref":"control_flow_macros.html#replace-non-branching-bar-with-bar"},{"type":"extras","title":"Move assignments from `with` statement head - Control Flow Macros (if, case, ...)","doc":"Just because any program _could_ be written entirely within the head of a `with` statement doesn't mean it should be!\n\nStyler moves assignments that aren't trapped between `<-` outside of the head. Combined with the non-pattern-matching replacement above, we get the following:\n\n```elixir\n# Given\nwith foo <- bar,\n     x = y,\n     :ok <- baz,\n     bop <- boop,\n     :ok <- blop,\n     foo <- bar,\n     :success = hope_this_works! do\n  :ok\nend\n# Styled:\nfoo = bar\nx = y\n\nwith :ok <- baz,\n     bop = boop,\n     :ok <- blop do\n  foo = bar\n  :success = hope_this_works!\n  :ok\nend\n```\n\n#","ref":"control_flow_macros.html#move-assignments-from-with-statement-head"},{"type":"extras","title":"Remove redundant final clause - Control Flow Macros (if, case, ...)","doc":"If the pattern of the final clause of the head is also the `with` statements `do` body, styler nixes the final match and makes the right hand side of the clause into the do body.\n\n```elixir\n# Given\nwith {:ok, a} <- foo(),\n     {:ok, b} <- bar(a) do\n  {:ok, b}\nelse\n  error -> error\nend\n# Styled:\nwith {:ok, a} <- foo() do\n  bar(a)\nend\n```\n\n#","ref":"control_flow_macros.html#remove-redundant-final-clause"},{"type":"extras","title":"Replace with `case` - Control Flow Macros (if, case, ...)","doc":"A `with` statement with a single clause in the head and `else` is a really just a `case` clause putting on airs.\n\n```elixir\n# Given:\nwith :ok <- foo do\n  :success\nelse\n  :fail -> :failure\n  error -> error\nend\n# Styled:\ncase foo do\n  :ok -> :success\n  :fail -> :failure\n  error -> error\nend\n```\n\n#","ref":"control_flow_macros.html#replace-with-case"},{"type":"extras","title":"Replace with `if` - Control Flow Macros (if, case, ...)","doc":"Given Styler rewrites trivial `case` to `if`, it shouldn't be a surprise that that same rule means that `with` can be rewritten to `if` in some cases.\n\n```elixir\n# Given:\nwith true <- foo(), bar <- baz() do\n  {:ok, bar}\nelse\n  _ -> :error\nend\n# Styled:\nif foo() do\n  bar = baz()\n  {:ok, bar}\nelse\n  :error\nend\n```","ref":"control_flow_macros.html#replace-with-if"},{"type":"extras","title":"Mix Configs (config/config.exs, ...)","doc":"# Mix Configs\n\nMix Config files have their config stanzas sorted. Similar to the sorting of aliases, this delivers consistency to an otherwise arbitrary world, and can even help catch bugs like configuring the same key multiple times.\n\nA file is considered a config file if\n\n1. its path matches `config/.*\\.exs` or `rel/overlays/.*\\.exs`\n2. the file has `import Config`\n\nOnce a file is detected as a mix config, its `config/2,3` stanzas are grouped and ordered like so:\n\n- group config stanzas separated by assignments (`x = y`) together\n- sort each group according to erlang term sorting\n- move all existing assignments between the config stanzas to above the stanzas (without changing their ordering)","ref":"mix_configs.html"},{"type":"extras","title":"Examples - Mix Configs (config/config.exs, ...)","doc":"TODOs","ref":"mix_configs.html#examples"},{"type":"extras","title":"Module Directives (use, alias, ...)","doc":"","ref":"module_directives.html"},{"type":"extras","title":"Adds Moduledoc - Module Directives (use, alias, ...)","doc":"Adds `@moduledoc false` to modules without a moduledoc unless the module's name ends with one of the following:\n\n* `Test`\n* `Mixfile`\n* `MixProject`\n* `Controller`\n* `Endpoint`\n* `Repo`\n* `Router`\n* `Socket`\n* `View`\n* `HTML`\n* `JSON`","ref":"module_directives.html#adds-moduledoc"},{"type":"extras","title":"Directive Expansion - Module Directives (use, alias, ...)","doc":"Expands `Module.{SubmoduleA, SubmoduleB}` to their explicit forms for ease of searching.\n\n```elixir\n# Before\nimport Foo.{Bar, Baz, Bop}\nalias Foo.{Bar, Baz.A, Bop}\n\n# After\nimport Foo.Bar\nimport Foo.Baz\nimport Foo.Bop\n\nalias Foo.Bar\nalias Foo.Baz.A\nalias Foo.Bop\n```","ref":"module_directives.html#directive-expansion"},{"type":"extras","title":"Directive Organization - Module Directives (use, alias, ...)","doc":"Modules directives are sorted into the following order:\n\n* `@shortdoc`\n* `@moduledoc` (adds `@moduledoc false`)\n* `@behaviour`\n* `use`\n* `import` (sorted alphabetically)\n* `alias` (sorted alphabetically)\n* `require` (sorted alphabetically)\n* everything else (order unchanged)\n\n#","ref":"module_directives.html#directive-organization"},{"type":"extras","title":"Before - Module Directives (use, alias, ...)","doc":"```elixir\ndefmodule Foo do\n  @behaviour Lawful\n  alias A.A\n  require A\n\n  use B\n\n  def c(x), do: y\n\n  import C\n  @behaviour Chaotic\n  @doc \"d doc\"\n  def d do\n    alias X.X\n    alias H.H\n\n    alias Z.Z\n    import Ecto.Query\n    X.foo()\n  end\n  @shortdoc \"it's pretty short\"\n  import A\n  alias C.C\n  alias D.D\n\n  require C\n  require B\n\n  use A\n\n  alias C.C\n  alias A.A\n\n  @moduledoc \"README.md\"\n             |> File.read!()\n             |> String.split(\"<!-- MDOC !-->\")\n             |> Enum.fetch!(1)\nend\n```\n\n#","ref":"module_directives.html#before"},{"type":"extras","title":"After - Module Directives (use, alias, ...)","doc":"```elixir\ndefmodule Foo do\n  @shortdoc \"it's pretty short\"\n  @moduledoc \"README.md\"\n             |> File.read!()\n             |> String.split(\"<!-- MDOC !-->\")\n             |> Enum.fetch!(1)\n  @behaviour Chaotic\n  @behaviour Lawful\n\n  use B\n  use A.A\n\n  import A.A\n  import C\n\n  alias A.A\n  alias C.C\n  alias D.D\n\n  require A\n  require B\n  require C\n\n  def c(x), do: y\n\n  @doc \"d doc\"\n  def d do\n    import Ecto.Query\n\n    alias H.H\n    alias X.X\n    alias Z.Z\n\n    X.foo()\n  end\nend\n```\n\nIf any line previously relied on an alias, the alias is fully expanded when it is moved above the alias:\n\n```elixir\n# Given\nalias Foo.Bar\nimport Bar\n# Styled\nimport Foo.Bar\n\nalias Foo.Bar\n```","ref":"module_directives.html#after"},{"type":"extras","title":"Alias Lifting - Module Directives (use, alias, ...)","doc":"When a module with three parts is referenced two or more times, styler creates a new alias for that module and uses it.\n\n```elixir\n# Given\nrequire A.B.C\n\nA.B.C.foo()\nA.B.C.bar()\n\n# Styled\nalias A.B.C\n\nrequire C\n\nC.foo()\nC.bar()\n```\n\n#","ref":"module_directives.html#alias-lifting"},{"type":"extras","title":"Collisions - Module Directives (use, alias, ...)","doc":"Styler won't lift aliases that will collide with existing aliases, and likewise won't lift any module whose name would collide with a standard library name.\n\nYou can specify additional modules to exlude from lifting via the `:alias_lifting_exclude` configuration option. For the example above, the following configuration would keep Styler from creating the `alias A.B.C` node:\n\n```elixir\n# .formatter.exs\n[\n  plugins: [Styler],\n  styler: [alias_lifting_exclude: [:C]],\n]\n```","ref":"module_directives.html#collisions"},{"type":"extras","title":"Styler & Credo","doc":"#","ref":"credo.html"},{"type":"extras","title":"Credo Rules Styler Replaces - Styler & Credo","doc":"If you're using Credo and Styler, **we recommend disabling these rules in `.credo.exs`** to save on unnecessary checks in CI.\nAs long as you're running `mix format --check-formatted` in CI, Styler will be enforcing the rules for you, so checking them with Credo is redundant.\n\nDisabling the rules means updating your `.credo.exs` depending on your configuration:\n\n- if you're using `checks: %{enabled: [...]}`, ensure none of the checks are listed in your enabled checks\n- if you're using `checks: %{disabled: [...]}`, copy/paste the snippet below into the list\n- if you're using `checks: [...]`, copy/paste the snippet below into the list and ensure none of the checks appear earlier in the list\n\n```elixir\n# Styler Rewrites\n#\n# The following rules are automatically rewritten by Styler and so disabled here to save time\n# Some of the rules have `priority: :high`, meaning Credo runs them unless we explicitly disable them\n# (removing them from this file wouldn't be enough, the `false` is required)\n#\n{Credo.Check.Consistency.MultiAliasImportRequireUse, false},\n{Credo.Check.Consistency.ParameterPatternMatching, false},\n{Credo.Check.Design.AliasUsage, false},\n{Credo.Check.Readability.AliasOrder, false},\n{Credo.Check.Readability.BlockPipe, false},\n{Credo.Check.Readability.LargeNumbers, false},\n{Credo.Check.Readability.ModuleDoc, false},\n{Credo.Check.Readability.MultiAlias, false},\n{Credo.Check.Readability.OneArityFunctionInPipe, false},\n{Credo.Check.Readability.ParenthesesOnZeroArityDefs, false},\n{Credo.Check.Readability.PipeIntoAnonymousFunctions, false},\n{Credo.Check.Readability.PreferImplicitTry, false},\n{Credo.Check.Readability.SinglePipe, false},\n{Credo.Check.Readability.StrictModuleLayout, false},\n{Credo.Check.Readability.StringSigils, false},\n{Credo.Check.Readability.UnnecessaryAliasExpansion, false},\n{Credo.Check.Readability.WithSingleClause, false},\n{Credo.Check.Refactor.CaseTrivialMatches, false},\n{Credo.Check.Refactor.CondStatements, false},\n{Credo.Check.Refactor.FilterCount, false},\n{Credo.Check.Refactor.MapInto, false},\n{Credo.Check.Refactor.MapJoin, false},\n{Credo.Check.Refactor.NegatedConditionsInUnless, false},\n{Credo.Check.Refactor.NegatedConditionsWithElse, false},\n{Credo.Check.Refactor.PipeChainStart, false},\n{Credo.Check.Refactor.RedundantWithClauseResult, false},\n{Credo.Check.Refactor.UnlessWithElse, false},\n{Credo.Check.Refactor.WithClauses, false},\n ```","ref":"credo.html#credo-rules-styler-replaces"}],"content_type":"text/markdown"}